
.onoffswitch {
    display: inline;
    position: relative; width: 167px;
    margin-top:8px;
    -webkit-user-select:none; -moz-user-select:none; -ms-user-select: none;
}
.onoffswitch-checkbox {
    display: none;
}
.onoffswitch-label {
    display: block; overflow: hidden; cursor: pointer;
    border: 2px solid #999999; border-radius: 20px;
    margin:0;
}
.onoffswitch-inner {
    display: block; width: 200%; margin-left: -100%;
    transition: margin 0.3s ease-in 0s;
}
.onoffswitch-inner:before, .onoffswitch-inner:after {
    display: block; float: left; width: 50%; height: 30px; padding: 0; line-height: 30px;
    font-size: 15px; color: white; font-family: Trebuchet, Arial, sans-serif; font-weight: bold;
    box-sizing: border-box;
}
.onoffswitch-inner:before {
    content: "Hide Solution";
    padding-left: 10px;
    background-color: #34A7C1; color: #FFFFFF;
   }

.onoffswitch-inner:after {
    content: "Show Solution";
    padding-right: 10px;
    background-color: #73FA7E; color: #999999;
    text-align: right;
}
.onoffswitch-switch {
    display: block; width: 14px; margin: 6px;
    padding-top: 0px;
    background: #FFFFFF;
    position: absolute; top: 0; bottom: 0;
    text-align: center;
    vertical-align: middle;
    right: 133px;
    border: 2px solid #999999; border-radius: 20px;
    transition: all 0.25s ease-in 0s; 
}
.onoffswitch-checkbox:checked + .onoffswitch-label .onoffswitch-inner {
    margin-left: 0;
}
.onoffswitch-checkbox:checked + .onoffswitch-label .onoffswitch-switch {
    right: 0px; 
}

